from gw2apy.endpoints.endpoint import AbstractEndpoint


class Mounts(AbstractEndpoint):
    def __init__(self, client, category: str):
        super(Mounts, self).__init__(client=client)

        assert category in ["skins", "types"]
        self.endpoint_path = f"/v2/mounts/{category}"

        self.has_pages = True
        self.has_ids = True
        self.has_all_ids = True


class MountsSkins(Mounts):
    def __init__(self, client):
        super(MountsSkins, self).__init__(
            client=client,
            category="skins",
        )


class MountsTypes(Mounts):
    def __init__(self, client):
        super(MountsTypes, self).__init__(
            client=client,
            category="types",
        )



from gw2apy.endpoints.endpoint import AbstractEndpoint


class Recipes(AbstractEndpoint):
    def __init__(self, client):
        super(Recipes, self).__init__(client=client)

        self.endpoint_path = "/v2/recipes"

        self.has_pages = True
        self.has_ids = True


class RecipesSearch(AbstractEndpoint):
    def __init__(self, client):
        super(RecipesSearch, self).__init__(client=client)

        self.endpoint_path = "/v2/recipes/search"

    def get(self, **kwargs):
        input = kwargs.get("input")
        output = kwargs.get("output")

        c_input = input is not None
        c_output = output is not None

        c = c_input != c_output
        assert c, "Must provide either 'input' or 'output' exclusively."

        if c_input:
            path = f"?input={input}"
        else:
            path = f"?output={output}"
        return super(RecipesSearch, self).get(path=path)



from .account import (
    Account,
    AccountAchievements,
    AccountBank,
    AccountBuildStorage,
    AccountDailyCrafting,
    AccountDungeons,
    AccountDyes,
    AccountEmotes,
    AccountFinishers,
    AccountGliders,
    AccountHome,
    AccountHomeCats,
    AccountHomeNodes,
    AccountInventory,
    AccountLuck,
    AccountMailCarriers,
    AccountMapChests,
    AccountMasteries,
    AccountMasteryPoints,
    AccountMaterials,
    AccountMinis,
    AccountMounts,
    AccountMountsSkins,
    AccountMountsTypes,
    AccountNovelties,
    AccountOutfits,
    AccountPvPHeroes,
    AccountRaids,
    AccountRecipes,
    AccountSkins,
    AccountTitles,
    AccountWallet,
    AccountWorldBosses,
)
from .achievements import (
    Achievements,
    AchievementsCategories,
    AchievementsDaily,
    AchievementsDailyTomorrow,
    AchievementsGroups,
)
from .backstory import (
    Backstory,
    BackstoryAnswers,
    BackstoryQuestions,
)
from .build import (
    Build,
)
from .characters import (
    Characters,
    CharactersBackstory,
    CharactersCore,
    CharactersCrafting,
    CharactersEquipment,
    CharactersHeroPoints,
    CharactersInventory,
    CharactersRecipes,
    CharactersSAB,
    CharactersSkills,
    CharactersSpecializations,
    CharactersSubEndpoint,
    CharactersTraining,
)
from .colors import (
    Colors,
)
from .commerce import (
    CommerceDelivery,
    CommerceExchange,
    CommerceExchangeCoins,
    CommerceExchangeGems,
    CommerceListings,
    CommercePrices,
    CommerceTransaction,
    CommerceTransactionCurrentBuys,
    CommerceTransactionCurrentSells,
    CommerceTransactionHistoryBuys,
    CommerceTransactionHistorySells,
)
from .continents import (
    Continents,
)
from .currencies import (
    Currencies,
)
from .dailycrafting import (
    DailyCrafting,
)
from .dungeons import (
    Dungeons,
)
from .emblem import (
    Emblem,
    EmblemBackgrounds,
    EmblemForegrounds,
)
from .emotes import (
    Emotes,
)
from .endpoint import (
    AbstractEndpoint,
    AuthenticatedAbstractEndpoint,
)
from .files import (
    Files,
)
from .finishers import (
    Finishers,
)
from .gliders import (
    Gliders,
)
from .guild import (
    Guild,
)
from .home import (
    Home,
    HomeCats,
    HomeNodes,
)
from .items import (
    Items,
)
from .itemstats import (
    ItemStats,
)
from .legends import (
    Legends,
)
from .mailcarriers import (
    MailCarriers,
)
from .mapchests import (
    MapChests,
)
from .maps import (
    Maps,
)
from .masteries import (
    Masteries,
)
from .materials import (
    Materials,
)
from .minis import (
    Minis,
)
from .mounts import (
    Mounts,
    MountsSkins,
    MountsTypes,
)
from .novelties import (
    Novelties,
)
from .outfits import (
    Outfits,
)
from .pets import (
    Pets,
)
from .professions import (
    Professions,
)
from .pvp import (
    PvP,
    PvPAmulets,
    PvPGames,
    PvPHeroes,
    PvPRanks,
    PvPSeasons,
    PvPStandings,
    PvPStats,
)
from .quaggans import (
    Quaggans,
)
from .quests import (
    Quests,
)
from .races import (
    Races,
)
from .raids import (
    Raids,
)
from .recipes import (
    Recipes,
    RecipesSearch,
)
from .skills import (
    Skills,
)
from .skins import (
    Skins,
)
from .specializations import (
    Specializations,
)
from .stories import (
    Stories,
    StoriesSeasons,
)
from .titles import (
    Titles,
)
from .tokeninfo import (
    TokenInfo,
)
from .traits import (
    Traits,
)
from .worldbosses import (
    WorldBosses,
)
from .worlds import (
    Worlds,
)
from .wvw import (
    WvW,
    WvWAbilities,
    WvWMatches,
    WvWObjectives,
    WvWRanks,
    WvWUpgrades,
)



from typing import Iterable
import sys
from requests_futures.sessions import FuturesSession
from concurrent.futures import as_completed


class Client(object):
    def __init__(
        self,
        base_url: str = "https://api.guildwars2.com",
        schema_version: str = "2020-11-17T00:30:00.000Z",
        lang: str = "en",
        api_key: str = None,
        max_workers: int = 64,
        debug: bool = False,
    ):
        self.base_url = base_url
        self.schema_version = schema_version
        self.lang = lang

        self.api_key = api_key

        self.session = self.build_session(max_workers=max_workers)

        self.debug = debug

    def build_session(self, max_workers: int):
        session = FuturesSession(max_workers=max_workers)

        session.verify = True

        headers = {
            "User-Agent": "GuildWars2-API-Python-Wrapper",
            "Accept": "application/json",
            "Accept-Language": self.lang,
            "X-Schema-Version": self.schema_version,
        }

        if self.api_key is not None:
            headers["Authorization"] = f"Bearer {self.api_key}"

        session.headers.update(headers)
        return session

    def get(self, url: str):
        future = self.session.get(url, hooks={"response": JSONify})
        response = future.result()

        if self.debug:
            print(response.status_code, response.url, file=sys.stderr)

        return response

    def gets(self, urls: Iterable[str]):
        futures = [self.session.get(url, hooks={"response": JSONify}) for url in urls]
        responses = [future.result() for future in as_completed(futures)]

        if self.debug:
            for response in responses:
                print(response.status_code, response.url, file=sys.stderr)

        return responses


def JSONify(response, *args, **kwargs):
    response.data = response.json()



from gw2apy.endpoints.endpoint import AbstractEndpoint


class Backstory(AbstractEndpoint):
    def __init__(self, client, category: str):
        super(Backstory, self).__init__(client=client)

        assert category in ["answers", "questions"]
        self.endpoint_path = f"/v2/backstory/{category}"

        self.has_pages = True
        self.has_ids = True
        self.has_all_ids = True


class BackstoryAnswers(Backstory):
    def __init__(self, client):
        super(BackstoryAnswers, self).__init__(
            client=client,
            category="answers",
        )


class BackstoryQuestions(Backstory):
    def __init__(self, client):
        super(BackstoryQuestions, self).__init__(
            client=client,
            category="questions",
        )



from gw2apy.endpoints.endpoint import AbstractEndpoint


class PvP(AbstractEndpoint):
    def __init__(self, client, category: str):
        super(PvP, self).__init__(client=client)

        assert category in [
            "amulets",
            "games",
            "heroes",
            "ranks",
            "seasons",
            "standings",
            "stats",
        ]
        self.endpoint_path = f"/v2/pvp/{category}"

        self.has_pages = True
        self.has_ids = True
        self.has_all_ids = True


class PvPAmulets(PvP):
    def __init__(self, client):
        super(PvPAmulets, self).__init__(
            client=client,
            category="amulets",
        )


class PvPGames(PvP):
    def __init__(self, client):
        assert client.api_key is not None
        super(PvPGames, self).__init__(
            client=client,
            category="games",
        )


class PvPHeroes(PvP):
    def __init__(self, client):
        super(PvPHeroes, self).__init__(
            client=client,
            category="heroes",
        )


class PvPRanks(PvP):
    def __init__(self, client):
        super(PvPRanks, self).__init__(
            client=client,
            category="ranks",
        )


class PvPSeasons(PvP):
    def __init__(self, client):
        super(PvPSeasons, self).__init__(
            client=client,
            category="seasons",
        )


class PvPStandings(PvP):
    def __init__(self, client):
        assert client.api_key
        super(PvPStandings, self).__init__(
            client=client,
            category="standings",
        )


class PvPStats(PvP):
    def __init__(self, client):
        assert client.api_key
        super(PvPStats, self).__init__(
            client=client,
            category="stats",
        )



from gw2apy.endpoints.endpoint import AbstractEndpoint


class Build(AbstractEndpoint):
    def __init__(self, client):
        super(Build, self).__init__(client=client)

        self.endpoint_path = "/v2/build"



from itertools import chain


class AbstractEndpoint(object):
    def __init__(self, client):
        self.client = client

        self.endpoint_path = None

        self.has_pages = False
        self.has_ids = False
        self.has_all_ids = False

        self.max_page_size = 200

    def endpoint(self):
        response = self._endpoint()
        return response.data

    def path(self, path):
        response = self._path(path=path)
        return response.data

    def id(self, id):
        assert self.has_ids, f"{self.__class__} does not have ids."
        response = self._id(id=id)
        return response.data

    def page(self, page, page_size=None):
        assert self.has_pages, f"{self.__class__} does not have pages."
        response = self._page(page=page, page_size=page_size)
        return response.data

    def ids(self, ids):
        assert self.has_ids, f"{self.__class__} does not have ids."
        responses = self._ids(ids=ids)
        return flatten([response.data for response in responses])

    def pages(self, pages, page_size=None):
        assert self.has_pages, f"{self.__class__} does not have pages."
        responses = self._pages(pages=pages, page_size=page_size)
        return flatten([response.data for response in responses])

    def all(self):
        c = self.has_ids or self.has_pages
        assert c, f"{self.__class__} does not have ids nor pages."
        if self.has_all_ids:
            response = self._all()
            return response.data
        elif self.has_pages:
            responses = self._all_by_pages()
            return flatten([response.data for response in responses])
        else:
            responses = self._all_by_ids()
            return flatten([response.data for response in responses])

    def get(self, **kwargs):
        path = kwargs.get("path")
        id = kwargs.get("id")
        page = kwargs.get("page")
        ids = kwargs.get("ids")
        pages = kwargs.get("pages")
        page_size = kwargs.get("page_size")

        c_path = path is not None
        c_id = id is not None
        c_page = page is not None
        c_ids = ids is not None
        c_pages = pages is not None
        c_page_size = page_size is not None

        c = (c_path + c_id + c_page + c_ids + c_pages) > 1
        assert not c, "'path', 'id', 'page', 'ids', 'pages' are to be used separately."
        c = c_page_size and not (c_page or c_pages)
        assert not c, "'page_size' cannot be used without 'page' or 'pages'."

        if c_path:
            return self.path(path=path)
        elif c_id:
            return self.id(id=id)
        elif c_page:
            return self.page(page=page, page_size=page_size)
        elif c_ids:
            if ids != "all":
                return self.ids(ids=ids)
            else:
                return self.all()
        elif c_pages:
            return self.pages(pages=pages, page_size=page_size)
        else:
            return self.endpoint()

    def _endpoint(self):
        url = self._endpoint_base_url()
        response = self.client.get(url=url)
        return response

    def _path(self, path):
        suffix = f"{path}"
        url = self._endpoint_base_url() + suffix
        response = self.client.get(url=url)
        return response

    def _id(self, id):
        suffix = f"?id={id}"
        url = self._endpoint_base_url() + suffix
        response = self.client.get(url=url)
        return response

    def _page(self, page, page_size=None):
        if page_size is None:
            page_size = self.max_page_size
        assert 0 <= page_size <= self.max_page_size

        suffix = f"?page={page}&page_size={page_size}"
        url = self._endpoint_base_url() + suffix
        response = self.client.get(url=url)
        return response

    def _ids(self, ids):
        chunked_ids = [c for c in chunk(ids, self.max_page_size)]
        suffixes = [f"?ids={','.join(map(str, c))}" for c in chunked_ids]
        urls = [self._endpoint_base_url() + suffix for suffix in suffixes]
        responses = self.client.gets(urls=urls)
        return responses

    def _pages(self, pages, page_size=None):
        if page_size is None:
            page_size = self.max_page_size
        assert 0 <= page_size <= self.max_page_size

        suffixes = [f"?page={page}&page_size={page_size}" for page in pages]
        urls = [self._endpoint_base_url() + suffix for suffix in suffixes]
        responses = self.client.gets(urls=urls)
        return responses

    def _all(self):
        suffix = "?ids=all"
        url = self._endpoint_base_url() + suffix
        response = self.client.get(url=url)
        return response

    def _all_by_ids(self):
        response = self._endpoint()
        ids = response.data
        response = self._ids(ids=ids)
        return response

    def _all_by_pages(self):
        first_response = self._page(page=0)
        num_pages = int(first_response.headers["X-Page-Total"])
        remaining_responses = self._pages(pages=range(1, num_pages))
        return [first_response] + remaining_responses

    def _endpoint_base_url(self):
        return self.client.base_url + self.endpoint_path


class AuthenticatedAbstractEndpoint(AbstractEndpoint):
    def __init__(self, client):
        assert client.api_key is not None
        super(AuthenticatedAbstractEndpoint, self).__init__(client=client)


def chunk(l, n):
    for i in range(0, len(l), n):
        yield l[i : i + n]


def flatten(l):
    return list(chain(*l))



from gw2apy.endpoints.endpoint import AuthenticatedAbstractEndpoint


class Guild(AuthenticatedAbstractEndpoint):
    def __init__(self, client):
        super(Guild, self).__init__(client=client)

        self.endpoint_path = f"/v2/guild"

    def get(self, **kwargs):
        search = kwargs.get("search")

        c_search = search is not None

        if c_search:
            path = f"?search={search}"
            return super(Guild, self).get(path=path)
        else:
            return super(Guild, self).get(kwargs=kwargs)



from gw2apy.endpoints.endpoint import AbstractEndpoint


class Emblem(AbstractEndpoint):
    def __init__(self, client, category: str):
        super(Emblem, self).__init__(client=client)

        assert category in ["backgrounds", "foregrounds"]
        self.endpoint_path = f"/v2/emblem/{category}"

        self.has_pages = True
        self.has_ids = True


class EmblemBackgrounds(Emblem):
    def __init__(self, client):
        super(EmblemBackgrounds, self).__init__(
            client=client,
            category="backgrounds",
        )


class EmblemForegrounds(Emblem):
    def __init__(self, client):
        super(EmblemForegrounds, self).__init__(
            client=client,
            category="foregrounds",
        )



from gw2apy.endpoints.endpoint import AbstractEndpoint


class Home(AbstractEndpoint):
    def __init__(self, client, category: str):
        super(Home, self).__init__(client=client)

        assert category in ["cats", "nodes"]
        self.endpoint_path = f"/v2/home/{category}"

        self.has_pages = True
        self.has_ids = True
        self.has_all_ids = True


class HomeCats(Home):
    def __init__(self, client):
        super(HomeCats, self).__init__(
            client=client,
            category="cats",
        )


class HomeNodes(Home):
    def __init__(self, client):
        super(HomeNodes, self).__init__(
            client=client,
            category="nodes",
        )



from gw2apy.endpoints.endpoint import AbstractEndpoint, AuthenticatedAbstractEndpoint


class CommerceDelivery(AuthenticatedAbstractEndpoint):
    def __init__(self, client):
        super(CommerceDelivery, self).__init__(client=client)

        self.endpoint_path = "/v2/commerce/delivery"

        self.has_pages = True


class CommerceExchange(AbstractEndpoint):
    def __init__(self, client, currency: str):
        super(CommerceExchange, self).__init__(client=client)

        assert currency in ["coins", "gems"]
        self.endpoint_path = f"/v2/commerce/exchange/{currency}"

    def get(self, quantity: int):
        assert quantity >= 0
        path = f"?quantity={quantity}"
        return super(CommerceExchange, self).get(path=path)


class CommerceExchangeGems(CommerceExchange):
    def __init__(self, client):
        super(CommerceExchangeGems, self).__init__(
            client=client,
            currency="gems",
        )


class CommerceExchangeCoins(CommerceExchange):
    def __init__(self, client):
        super(CommerceExchangeCoins, self).__init__(
            client=client,
            currency="coins",
        )


class CommerceListings(AbstractEndpoint):
    def __init__(self, client):
        super(CommerceListings, self).__init__(client=client)

        self.endpoint_path = "/v2/commerce/listings"

        self.has_pages = True
        self.has_ids = True


class CommercePrices(AbstractEndpoint):
    def __init__(self, client):
        super(CommercePrices, self).__init__(client=client)

        self.endpoint_path = "/v2/commerce/prices"

        self.has_pages = True
        self.has_ids = True


class CommerceTransaction(AuthenticatedAbstractEndpoint):
    def __init__(self, client, category: str, order: str):
        super(CommerceTransaction, self).__init__(client=client)

        assert category in ["current", "history"]
        assert order in ["buys", "sells"]
        self.endpoint_path = f"/v2/commerce/transactions/{category}/{order}"


class CommerceTransactionCurrentBuys(CommerceTransaction):
    def __init__(self, client):
        super(CommerceTransactionCurrentBuys, self).__init__(
            client=client,
            category="current",
            order="buys",
        )


class CommerceTransactionCurrentSells(CommerceTransaction):
    def __init__(self, client):
        super(CommerceTransactionCurrentSells, self).__init__(
            client=client,
            category="current",
            order="sells",
        )


class CommerceTransactionHistoryBuys(CommerceTransaction):
    def __init__(self, client):
        super(CommerceTransactionHistoryBuys, self).__init__(
            client=client,
            category="history",
            order="buys",
        )


class CommerceTransactionHistorySells(CommerceTransaction):
    def __init__(self, client):
        super(CommerceTransactionHistorySells, self).__init__(
            client=client,
            category="history",
            order="sells",
        )



import unittest

from gw2apy.client import Client
from gw2apy.endpoints import CommercePrices


class SimpleTest(unittest.TestCase):
    def test_prices_id(self):
        client = Client()
        endpoint = CommercePrices(client=client)
        response = endpoint._endpoint()
        self.assertEqual(response.status_code, 200)


if __name__ == "__main__":
    unittest.main()



from gw2apy.endpoints.endpoint import AuthenticatedAbstractEndpoint


class TokenInfo(AuthenticatedAbstractEndpoint):
    def __init__(self, client):
        super(TokenInfo, self).__init__(client=client)

        self.endpoint_path = "/v2/tokeninfo"



from gw2apy.endpoints.endpoint import AbstractEndpoint


class Achievements(AbstractEndpoint):
    def __init__(self, client):
        super(Achievements, self).__init__(client=client)

        self.endpoint_path = "/v2/achievements"

        self.has_pages = True
        self.has_ids = True


class AchievementsCategories(AbstractEndpoint):
    def __init__(self, client):
        super(AchievementsCategories, self).__init__(client=client)

        self.endpoint_path = "/v2/achievements/categories"

        self.has_pages = True
        self.has_ids = True
        self.has_all_ids = True


class AchievementsDaily(AbstractEndpoint):
    def __init__(self, client):
        super(AchievementsDaily, self).__init__(client=client)

        self.endpoint_path = "/v2/achievements/daily"


class AchievementsDailyTomorrow(AbstractEndpoint):
    def __init__(self, client):
        super(AchievementsDailyTomorrow, self).__init__(client=client)

        self.endpoint_path = "/v2/achievements/daily/tomorrow"


class AchievementsGroups(AbstractEndpoint):
    def __init__(self, client):
        super(AchievementsGroups, self).__init__(client=client)

        self.endpoint_path = "/v2/achievements/groups"

        self.has_pages = True
        self.has_ids = True
        self.has_all_ids = True



from pathlib import Path
from setuptools import setup, find_packages


root = Path(__file__).parent.resolve()
with (root / "README.md").open() as file:
    long_description = file.read()
with (root / "requirements.txt").open() as file:
    install_requires = [l.strip() for l in file.readlines()]

setup(
    name="gw2apy",
    version="0.0.1",
    description="Guild Wars 2 API Python Wrapper",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Guillaume Rochette",
    author_email="rochette.guillaume@gmail.com",
    url="https://github.com/GuillaumeRochette/gw2apy/",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
        "Natural Language :: English",
    ],
    packages=find_packages(),
    python_requires=">=3.6",
    install_requires=install_requires,
    extras_require={
        "dev": ["check-manifest"],
        "test": ["coverage"],
    },
    project_urls={
        "Bug Reports": "https://github.com/GuillaumeRochette/gw2apy/issues",
        "Source": "https://github.com/GuillaumeRochette/gw2apy/",
    },
    options={"bdist_wheel": {"universal": True}},
)



from gw2apy.endpoints.endpoint import AbstractEndpoint


class WvW(AbstractEndpoint):
    def __init__(self, client, category: str):
        super(WvW, self).__init__(client=client)

        assert category in [
            "abilities",
            "matches",
            "objectives",
            "ranks",
            "upgrades",
        ]
        self.endpoint_path = f"/v2/wvw/{category}"

        self.has_pages = True
        self.has_ids = True
        self.has_all_ids = True


class WvWAbilities(WvW):
    def __init__(self, client):
        super(WvWAbilities, self).__init__(
            client=client,
            category="abilities",
        )


class WvWMatches(WvW):
    def __init__(self, client):
        super(WvWMatches, self).__init__(
            client=client,
            category="matches",
        )


class WvWObjectives(WvW):
    def __init__(self, client):
        super(WvWObjectives, self).__init__(
            client=client,
            category="objectives",
        )


class WvWRanks(WvW):
    def __init__(self, client):
        super(WvWRanks, self).__init__(
            client=client,
            category="ranks",
        )


class WvWUpgrades(WvW):
    def __init__(self, client):
        super(WvWUpgrades, self).__init__(
            client=client,
            category="upgrades",
        )



from gw2apy.endpoints.endpoint import AuthenticatedAbstractEndpoint


class Characters(AuthenticatedAbstractEndpoint):
    def __init__(self, client):
        super(Characters, self).__init__(client=client)

        self.endpoint_path = "/v2/characters"

        self.has_pages = True
        self.has_ids = True
        self.has_all_ids = True


class CharactersSubEndpoint(AuthenticatedAbstractEndpoint):
    def __init__(self, client, category: str):
        super(CharactersSubEndpoint, self).__init__(client=client)

        assert category in [
            "backstory",
            "core",
            "crafting",
            "equipment",
            "heropoints",
            "inventory",
            "recipes",
            "sab",
            "skills",
            "specializations",
            "training",
        ]
        self.endpoint_path = f"/v2/characters"
        self.category = category

    def get(self, character_name: str):
        path = f"/{character_name}/{self.category}"
        return super(CharactersSubEndpoint, self).get(path=path)


class CharactersBackstory(CharactersSubEndpoint):
    def __init__(self, client):
        super(CharactersBackstory, self).__init__(
            client=client,
            category="backstory",
        )


class CharactersCore(CharactersSubEndpoint):
    def __init__(self, client):
        super(CharactersCore, self).__init__(
            client=client,
            category="core",
        )


class CharactersCrafting(CharactersSubEndpoint):
    def __init__(self, client):
        super(CharactersCrafting, self).__init__(
            client=client,
            category="crafting",
        )


class CharactersEquipment(CharactersSubEndpoint):
    def __init__(self, client):
        super(CharactersEquipment, self).__init__(
            client=client,
            category="equipment",
        )


class CharactersHeroPoints(CharactersSubEndpoint):
    def __init__(self, client):
        super(CharactersHeroPoints, self).__init__(
            client=client,
            category="heropoints",
        )


class CharactersInventory(CharactersSubEndpoint):
    def __init__(self, client):
        super(CharactersInventory, self).__init__(
            client=client,
            category="inventory",
        )


class CharactersRecipes(CharactersSubEndpoint):
    def __init__(self, client):
        super(CharactersRecipes, self).__init__(
            client=client,
            category="recipes",
        )


class CharactersSAB(CharactersSubEndpoint):
    def __init__(self, client):
        super(CharactersSAB, self).__init__(
            client=client,
            category="sab",
        )


class CharactersSkills(CharactersSubEndpoint):
    def __init__(self, client):
        super(CharactersSkills, self).__init__(
            client=client,
            category="skills",
        )


class CharactersSpecializations(CharactersSubEndpoint):
    def __init__(self, client):
        super(CharactersSpecializations, self).__init__(
            client=client,
            category="specializations",
        )


class CharactersTraining(CharactersSubEndpoint):
    def __init__(self, client):
        super(CharactersTraining, self).__init__(
            client=client,
            category="training",
        )



from gw2apy.endpoints.endpoint import AuthenticatedAbstractEndpoint


class Account(AuthenticatedAbstractEndpoint):
    def __init__(self, client):
        super(Account, self).__init__(client=client)

        self.endpoint_path = "/v2/account"


class AccountAchievements(AuthenticatedAbstractEndpoint):
    def __init__(self, client):
        super(AccountAchievements, self).__init__(client=client)

        self.endpoint_path = "/v2/account/achievements"


class AccountBank(AuthenticatedAbstractEndpoint):
    def __init__(self, client):
        super(AccountBank, self).__init__(client=client)

        self.endpoint_path = "/v2/account/bank"


class AccountBuildStorage(AuthenticatedAbstractEndpoint):
    def __init__(self, client):
        super(AccountBuildStorage, self).__init__(client=client)

        self.endpoint_path = "/v2/account/buildstorage"


class AccountDailyCrafting(AuthenticatedAbstractEndpoint):
    def __init__(self, client):
        super(AccountDailyCrafting, self).__init__(client=client)

        self.endpoint_path = "/v2/account/dailycrafting"


class AccountDungeons(AuthenticatedAbstractEndpoint):
    def __init__(self, client):
        super(AccountDungeons, self).__init__(client=client)

        self.endpoint_path = "/v2/account/dungeons"


class AccountDyes(AuthenticatedAbstractEndpoint):
    def __init__(self, client):
        super(AccountDyes, self).__init__(client=client)

        self.endpoint_path = "/v2/account/dyes"


class AccountEmotes(AuthenticatedAbstractEndpoint):
    def __init__(self, client):
        super(AccountEmotes, self).__init__(client=client)

        self.endpoint_path = "/v2/account/emotes"


class AccountFinishers(AuthenticatedAbstractEndpoint):
    def __init__(self, client):
        super(AccountFinishers, self).__init__(client=client)

        self.endpoint_path = "/v2/account/finishers"


class AccountGliders(AuthenticatedAbstractEndpoint):
    def __init__(self, client):
        super(AccountGliders, self).__init__(client=client)

        self.endpoint_path = "/v2/account/gliders"


class AccountHome(AuthenticatedAbstractEndpoint):
    def __init__(self, client, category: str):
        super(AccountHome, self).__init__(client=client)

        assert category in ["cats", "nodes"]
        self.endpoint_path = f"/v2/account/home/{category}"


class AccountHomeCats(AccountHome):
    def __init__(self, client):
        super(AccountHomeCats, self).__init__(
            client=client,
            category="cats",
        )


class AccountHomeNodes(AccountHome):
    def __init__(self, client):
        super(AccountHomeNodes, self).__init__(
            client=client,
            category="nodes",
        )


class AccountInventory(AuthenticatedAbstractEndpoint):
    def __init__(self, client):
        super(AccountInventory, self).__init__(client=client)

        self.endpoint_path = "/v2/account/inventory"


class AccountLuck(AuthenticatedAbstractEndpoint):
    def __init__(self, client):
        super(AccountLuck, self).__init__(client=client)

        self.endpoint_path = "/v2/account/luck"


class AccountMailCarriers(AuthenticatedAbstractEndpoint):
    def __init__(self, client):
        super(AccountMailCarriers, self).__init__(client=client)

        self.endpoint_path = "/v2/account/mailcarriers"


class AccountMapChests(AuthenticatedAbstractEndpoint):
    def __init__(self, client):
        super(AccountMapChests, self).__init__(client=client)

        self.endpoint_path = "/v2/account/mapchests"


class AccountMasteries(AuthenticatedAbstractEndpoint):
    def __init__(self, client):
        super(AccountMasteries, self).__init__(client=client)

        self.endpoint_path = "/v2/account/masteries"


class AccountMasteryPoints(AuthenticatedAbstractEndpoint):
    def __init__(self, client):
        super(AccountMasteryPoints, self).__init__(client=client)

        self.endpoint_path = "/v2/account/mastery/points"


class AccountMaterials(AuthenticatedAbstractEndpoint):
    def __init__(self, client):
        super(AccountMaterials, self).__init__(client=client)

        self.endpoint_path = "/v2/account/materials"


class AccountMinis(AuthenticatedAbstractEndpoint):
    def __init__(self, client):
        super(AccountMinis, self).__init__(client=client)

        self.endpoint_path = "/v2/account/minis"


class AccountMounts(AuthenticatedAbstractEndpoint):
    def __init__(self, client, category: str):
        super(AccountMounts, self).__init__(client=client)

        assert category in ["skins", "types"]
        self.endpoint_path = f"/v2/account/mounts/{category}"


class AccountMountsSkins(AccountMounts):
    def __init__(self, client):
        super(AccountMountsSkins, self).__init__(
            client=client,
            category="skins",
        )


class AccountMountsTypes(AccountMounts):
    def __init__(self, client):
        super(AccountMountsTypes, self).__init__(
            client=client,
            category="types",
        )


class AccountNovelties(AuthenticatedAbstractEndpoint):
    def __init__(self, client):
        super(AccountNovelties, self).__init__(client=client)

        self.endpoint_path = "/v2/account/novelties"


class AccountOutfits(AuthenticatedAbstractEndpoint):
    def __init__(self, client):
        super(AccountOutfits, self).__init__(client=client)

        self.endpoint_path = "/v2/account/outfits"


class AccountPvPHeroes(AuthenticatedAbstractEndpoint):
    def __init__(self, client):
        super(AccountPvPHeroes, self).__init__(client=client)

        self.endpoint_path = "/v2/account/pvp/heroes"


class AccountRaids(AuthenticatedAbstractEndpoint):
    def __init__(self, client):
        super(AccountRaids, self).__init__(client=client)

        self.endpoint_path = "/v2/account/raids"


class AccountRecipes(AuthenticatedAbstractEndpoint):
    def __init__(self, client):
        super(AccountRecipes, self).__init__(client=client)

        self.endpoint_path = "/v2/account/recipes"


class AccountSkins(AuthenticatedAbstractEndpoint):
    def __init__(self, client):
        super(AccountSkins, self).__init__(client=client)

        self.endpoint_path = "/v2/account/skins"


class AccountTitles(AuthenticatedAbstractEndpoint):
    def __init__(self, client):
        super(AccountTitles, self).__init__(client=client)

        self.endpoint_path = "/v2/account/titles"


class AccountWallet(AuthenticatedAbstractEndpoint):
    def __init__(self, client):
        super(AccountWallet, self).__init__(client=client)

        self.endpoint_path = "/v2/account/wallet"


class AccountWorldBosses(AuthenticatedAbstractEndpoint):
    def __init__(self, client):
        super(AccountWorldBosses, self).__init__(client=client)

        self.endpoint_path = "/v2/account/worldbosses"



from gw2apy.endpoints.endpoint import AbstractEndpoint


class Minis(AbstractEndpoint):
    def __init__(self, client):
        super(Minis, self).__init__(client=client)

        self.endpoint_path = "/v2/minis"

        self.has_pages = True
        self.has_ids = True
        self.has_all_ids = True
